package com.bizmatics.controller.web;


import com.bizmatics.common.core.bean.ApiResult;
import com.bizmatics.model.BulletinAlarmConfig;
import com.bizmatics.model.BulletinCruxConfig;
import com.bizmatics.model.DeviceAnalogVariableList;
import com.bizmatics.model.DeviceAttribute;
import com.bizmatics.model.vo.BulletinHistoricalConfigAddVo;
import com.bizmatics.service.BulletinCruxConfigService;
import com.bizmatics.service.aop.BusinessType;
import com.bizmatics.service.aop.Log;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 数据看板
 *
 * @author ya
 * @since 2022-06-01
 */
@RestController
@RequestMapping("/bulletinCruxConfig")
public class BulletinCruxConfigController {
    @Autowired
    private BulletinCruxConfigService bulletinCruxConfigService;

    /**
     * 数据看板-数据配置查询
     *
     * @param siteId    站点ID
     * @param id        配置id
     * @param colorType 类型
     * @return
     */
    @GetMapping("/getBulletinCruxConfigList")
    public ApiResult<List<Object>> getBulletinCruxConfigList(@RequestParam(value = "siteId") Integer siteId,
                                                             @RequestParam(value = "id", required = false, defaultValue = "0") Integer id,
                                                             @RequestParam(value = "colorType", required = false) String colorType) {
        return ApiResult.success(bulletinCruxConfigService.getBulletinCruxConfigList(siteId, id, colorType));
    }


    /**
     * 数据看板-数据配置1编辑新增
     *
     * @param bulletinCruxConfig
     * @return
     */
    @Log(title = "数据看板", businessType = BusinessType.INSERT)
    @PostMapping("editBulletinCruxConfig")
    public ApiResult<Void> editBulletinCruxConfig(@RequestBody BulletinCruxConfig bulletinCruxConfig) {
        bulletinCruxConfigService.editBulletinCruxConfig(bulletinCruxConfig);
        return ApiResult.success();
    }

    /**
     * 数据看板-数据配置2编辑新增
     *
     * @param bulletinHistoricalConfigAddVo
     * @return
     */
    @Log(title = "数据看板", businessType = BusinessType.INSERT)
    @PostMapping("editBulletinHistoricalConfig")
    public ApiResult<Void> editBulletinHistoricalConfig(@RequestBody BulletinHistoricalConfigAddVo bulletinHistoricalConfigAddVo) {
        bulletinCruxConfigService.editBulletinHistoricalConfig(bulletinHistoricalConfigAddVo);
        return ApiResult.success();
    }

    /**
     * 数据看板-数据配置3编辑新增
     *
     * @param bulletinAlarmConfig
     * @return
     */
    @Log(title = "数据看板", businessType = BusinessType.INSERT)
    @PostMapping("editBulletinAlarmConfig")
    public ApiResult<Void> editBulletinAlarmConfig(@RequestBody BulletinAlarmConfig bulletinAlarmConfig) {
        bulletinCruxConfigService.editBulletinAlarmConfig(bulletinAlarmConfig);
        return ApiResult.success();
    }

    /**
     * 数据看板-数据配置1删除
     *
     * @param id 配置ID
     * @return
     */
    @Log(title = "数据看板", businessType = BusinessType.DELETE)
    @GetMapping("/delBulletinCruxConfig")
    public ApiResult<Void> delBulletinCruxConfig(@RequestParam(value = "id") Integer id) {
        bulletinCruxConfigService.delBulletinCruxConfig(id);
        return ApiResult.success();
    }


    /**
     * 数据看板-数据配置2删除
     *
     * @param id 配置ID
     * @return
     */
    @Log(title = "数据看板", businessType = BusinessType.DELETE)
    @GetMapping("/delBulletinHistoricalConfig")
    public ApiResult<Void> delBulletinHistoricalConfig(@RequestParam(value = "id") Integer id) {
        bulletinCruxConfigService.delBulletinHistoricalConfig(id);
        return ApiResult.success();
    }


    /**
     * 数据看板-数据配置3删除
     *
     * @param id 配置ID
     * @return
     */
    @Log(title = "数据看板", businessType = BusinessType.DELETE)
    @GetMapping("/delBulletinAlarmConfig")
    public ApiResult<Void> delBulletinAlarmConfig(@RequestParam(value = "id") Integer id) {
        bulletinCruxConfigService.delBulletinAlarmConfig(id);
        return ApiResult.success();
    }


    /**
     * 监控设备下拉框
     *
     * @param siteId 站点id
     * @return
     */
    @GetMapping("/getDeviceAttribute")
    public ApiResult<List<DeviceAttribute>> getDeviceAttribute(@RequestParam(value = "siteId") Integer siteId) {
        return ApiResult.success(bulletinCruxConfigService.getDeviceAttribute(siteId));
    }


    /**
     * 变量下拉框
     *
     * @param monitoringEquipment 监控设备ID
     * @return
     */
    @GetMapping("/getDeviceAnalogVariableList")
    public ApiResult<List<DeviceAnalogVariableList>> getDeviceAnalogVariableList(@RequestParam(value = "monitoringEquipment") Integer monitoringEquipment) {
        return ApiResult.success(bulletinCruxConfigService.getDeviceAnalogVariableList(monitoringEquipment));
    }

}

