package com.bizmatics.controller.web;


import com.bizmatics.common.core.bean.ApiResult;
import com.bizmatics.common.core.bean.CommonPage;
import com.bizmatics.model.SysPlatform;
import com.bizmatics.model.SysTenant;
import com.bizmatics.model.vo.SysTenantOneVo;
import com.bizmatics.model.vo.SysTenantTwoVo;
import com.bizmatics.service.SysPlatformService;
import com.bizmatics.service.SysTenantService;
import com.bizmatics.service.aop.BusinessType;
import com.bizmatics.service.aop.Log;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 个人中心
 *
 * @author ya
 * @since 2022-05-05
 */
@RestController
@RequestMapping("/tenantConfig")
public class TenantConfigController {

    @Autowired
    private SysTenantService sysTenantService;

    @Autowired
    private SysPlatformService sysPlatformService;


    /**
     * 租户管理-租户单位配置-列表查询
     *
     * @param sysTenantOneVo
     * @return
     */
    @PostMapping("getTenantData")
    public ApiResult<CommonPage<SysTenantTwoVo>> getTenantData(@RequestBody SysTenantOneVo sysTenantOneVo) {
        return ApiResult.success(sysTenantService.getTenantData(sysTenantOneVo));
    }


    /**
     * 租户管理-租户单位配置-新增
     *
     * @param sysTenant
     * @return
     */
    @Log(title = "租户管理-单位管理", businessType = BusinessType.INSERT)
    @PostMapping("addTenantData")
    public ApiResult<Void> addTenantData(@RequestBody SysTenant sysTenant) {
        sysTenantService.addTenantData(sysTenant);
        return ApiResult.success();
    }

    /**
     * 租户管理-租户单位配置-修改
     *
     * @param sysTenant
     * @return
     */
    @Log(title = "租户管理-单位管理", businessType = BusinessType.UPDATE)
    @PostMapping("updateTenantData")
    public ApiResult<Void> updateTenantData(@RequestBody SysTenant sysTenant) {
        sysTenantService.updateTenantData(sysTenant);
        return ApiResult.success();
    }


    /**
     * 租户管理-租户单位配置-系统平台下拉框数据查询
     * @return
     */
    @GetMapping("/getPlatformBoxList")
    public ApiResult<List<SysPlatform>> getPlatformBoxList() {
        return ApiResult.success(sysPlatformService.getPlatformBoxList());
    }

}

