package com.bizmatics.controller.web.system;

import com.bizmatics.common.core.bean.ApiResult;
import com.bizmatics.common.mvc.utils.ServletUtils;
import com.bizmatics.model.SysTenantConfig;
import com.bizmatics.model.system.LoginStyle;
import com.bizmatics.model.system.SysMenu;
import com.bizmatics.model.system.SysUser;
import com.bizmatics.model.vo.RouterVo;
import com.bizmatics.service.SysTenantConfigService;
import com.bizmatics.service.config.security.LoginUser;
import com.bizmatics.service.system.ISysMenuService;
import com.bizmatics.service.system.ISysUserService;
import com.bizmatics.service.system.impl.SysLoginService;
import com.bizmatics.service.system.impl.SysPermissionService;
import com.bizmatics.service.system.impl.TokenService;
import com.bizmatics.service.vo.LoginBody;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;


/**
 * 登录验证
 * 
 * @author yq
 */
@RestController
public class SysLoginController
{
    @Autowired
    private SysLoginService loginService;

    @Autowired
    private ISysMenuService menuService;

    @Autowired
    private SysPermissionService permissionService;

    @Autowired
    private TokenService tokenService;

    @Autowired
    private SysTenantConfigService sysTenantConfigService;

    @Autowired
    private ISysUserService userService;

    /**
     * 登录方法
     * 
     * @param loginBody 登录信息
     * @return 结果
     */
    @PostMapping("/login")
    public ApiResult<String> login(@RequestBody LoginBody loginBody)
    {
        // 生成令牌
        String token = loginService.login(loginBody.getUsername(), loginBody.getPassword(), loginBody.getCode(),
                loginBody.getUuid());
        return ApiResult.success(token);
    }

    /**
     * 获取用户信息
     * 
     * @return 用户信息
     */
    @GetMapping("getInfo")
    public ApiResult<Map<String,Object>> getInfo()
    {
        LoginUser loginUser = tokenService.getLoginUser(ServletUtils.getRequest());
        SysUser user = loginUser.getUser();
        // 角色集合
        Set<String> roles = permissionService.getRolePermission(user);
        // 权限集合
        Set<String> permissions = permissionService.getMenuPermission(user);
        Map<String,Object> ajax = new HashMap<>();
        ajax.put("user", user);
        ajax.put("roles", roles);
        ajax.put("permissions", permissions);
        List<String> list = new ArrayList<>();
        list.add("READ");
        ajax.put("ability",list);
        return ApiResult.success(ajax);
    }

    /**
     * 获取路由信息
     * 
     * @return 路由信息
     */
    @GetMapping("getRouters")
    public ApiResult<List<RouterVo>> getRouters()
    {
        LoginUser loginUser = tokenService.getLoginUser(ServletUtils.getRequest());
        // 用户信息
        SysUser user = loginUser.getUser();
        List<SysMenu> menus = new ArrayList<>();
        SysUser sysUser = userService.selectUserById(user.getUserId());
        if (sysUser.getUserType().equals("01")){
            menus = menuService.selectMenuTreeBy(sysUser.getTenantId());
        }else{
            menus = menuService.selectMenuTreeByUserId(user.getUserId());
        }
        return ApiResult.success(menuService.buildMenusNew(menus));
    }

    /**
     * 登录页样式
     * @param url
     * @return
     */
    @GetMapping("/getLoginStyle")
    public ApiResult<LoginStyle> getLoginStyle(@RequestParam String url)
    {
        LoginStyle loginStyle = loginService.getLoginStyle(url);
        return ApiResult.success(loginStyle);
    }


//    /**
//     * 登录中间页-样式配置查询
//     *
//     * @param url 域名
//     * @return
//     */
//    @GetMapping("/getTenantConfig")
//    public ApiResult<List<SysTenantConfig>> getTenantConfig(@RequestParam String url) {
//        return ApiResult.success(sysTenantConfigService.getTenantConfig(url));
//    }
}
