package com.bizmatics.service.impl;

import com.bizmatics.common.core.exception.BusinessException;
import com.bizmatics.common.mvc.base.AbstractCrudService;
import com.bizmatics.model.SiteGrouping;
import com.bizmatics.model.system.SysUser;
import com.bizmatics.persistence.mapper.SiteGroupingMapper;
import com.bizmatics.persistence.mapper.system.SysUserMapper;
import com.bizmatics.service.SiteGroupingService;
import com.bizmatics.service.util.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;

/**
 * 站点分组
 *
 * @author ya
 * @since 2021-09-17
 */
@Service
public class SiteGroupingServiceImpl extends AbstractCrudService<SiteGroupingMapper, SiteGrouping> implements SiteGroupingService {
    @Autowired
    private SysUserMapper userMapper;

    @Override
    public void siteGroupingAdd(String groupingName) {
        List<SysUser> tenantDaya = userMapper.getTenantId(SecurityUtils.getLoginUser().getUser().getUserId());
        if (tenantDaya.size() < 0) {
            throw new BusinessException("无此租户，请联系管理员");
        }
        SysUser user = SecurityUtils.getLoginUser().getUser();
        SiteGrouping siteGrouping = new SiteGrouping();
        siteGrouping.setGroupingName(groupingName);
        siteGrouping.setCreator(user.getUserName());
        siteGrouping.setCreationTime(LocalDateTime.now());
        siteGrouping.setTenantId(tenantDaya.get(0).getTenantId());
        this.save(siteGrouping);
    }

    @Override
    public void siteGroupingUpdate(String groupingName, int id) {
        SiteGrouping siteGrouping = new SiteGrouping();
        siteGrouping.setGroupingName(groupingName);
        siteGrouping.setId(id);
        this.updateById(siteGrouping);
    }
}
