package com.bizmatics.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.bizmatics.common.core.exception.BusinessException;
import com.bizmatics.common.mvc.base.AbstractCrudService;
import com.bizmatics.common.mvc.utils.ServletUtils;
import com.bizmatics.model.DeviceAttribute;
import com.bizmatics.model.SysTenantConfig;
import com.bizmatics.model.system.SysMenu;
import com.bizmatics.model.system.SysUser;
import com.bizmatics.model.vo.SysTenantConfigVo;
import com.bizmatics.persistence.mapper.SysTenantConfigMapper;
import com.bizmatics.persistence.mapper.system.SysUserMapper;
import com.bizmatics.service.SysTenantConfigService;
import com.bizmatics.service.config.security.LoginUser;
import com.bizmatics.service.system.impl.TokenService;
import com.bizmatics.service.util.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 租户配置表 服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-05-06
 */
@Service
public class SysTenantConfigServiceImpl extends AbstractCrudService<SysTenantConfigMapper, SysTenantConfig> implements SysTenantConfigService {
    @Autowired
    private SysUserMapper userMapper;

    @Override
    public List<SysTenantConfigVo> getTenantConfig(String url) {
        List<SysTenantConfigVo> list = baseMapper.getTenantConfig(url);
        return list;
    }

    @Override
    public List<SysTenantConfig> getMiddleConfig() {
        List<SysUser> tenantDaya = userMapper.getTenantId(SecurityUtils.getUser().getUserId());
        if (tenantDaya.size()<0) {
            throw new BusinessException("无此租户，请联系管理员");
        }
        LambdaQueryWrapper<SysTenantConfig> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(SysTenantConfig::getTenantId, tenantDaya.get(0).getTenantId());
        List<SysTenantConfig> list = this.list(queryWrapper);
        return list;
    }


    @Override
    public List<SysTenantConfig> getMiddleConfigList(Integer tenantId) {
        LambdaQueryWrapper<SysTenantConfig> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(SysTenantConfig::getTenantId, tenantId);
        List<SysTenantConfig> list = this.list(queryWrapper);
        return list;
    }

    @Override
    public void addMiddleConfig(SysTenantConfig sysTenantConfig){
        sysTenantConfig.setCreateBy(SecurityUtils.getUser().getUserName());
        sysTenantConfig.setCreateTime(LocalDateTime.now());
        this.save(sysTenantConfig);
    }
    @Override
    public void updaMiddleConfig(SysTenantConfig sysTenantConfig){
        sysTenantConfig.setUpdateBy(SecurityUtils.getUser().getUserName());
        sysTenantConfig.setUpdateTime(LocalDateTime.now());
        this.updateById(sysTenantConfig);
    }


    @Override
    public List<SysMenu> getMenuBoxList(Integer tenantId) {
        List<SysMenu> list = baseMapper.getMenuBoxList(tenantId,"C");
        return list;
    }
}
