package com.bizmatics.service.impl;

import com.bizmatics.common.core.bean.CommonPage;
import com.bizmatics.common.mvc.base.AbstractCrudService;
import com.bizmatics.model.SysTenant;
import com.bizmatics.model.system.SysUser;
import com.bizmatics.model.vo.SysTenantOneVo;
import com.bizmatics.model.vo.SysTenantTwoVo;
import com.bizmatics.model.vo.SysTenantVo;
import com.bizmatics.persistence.mapper.SysTenantMapper;
import com.bizmatics.service.SysTenantService;
import com.bizmatics.service.util.SecurityUtils;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.UUID;

/**
 * <p>
 * 租户信息表 服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-05-05
 */
@Service
public class SysTenantServiceImpl extends AbstractCrudService<SysTenantMapper, SysTenant> implements SysTenantService {

    @Override
    public List<SysTenantVo> getUserData() {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        Integer userId = user.getUserId().intValue();
        List<SysTenantVo> list = baseMapper.getUserData(userId);
        return list;
    }

    /**
     * 租户管理-租户列表查询
     *
     * @param sysTenantOneVo
     * @return
     */
    @Override
    public CommonPage<SysTenantTwoVo> getTenantData(SysTenantOneVo sysTenantOneVo) {
        if (sysTenantOneVo.getStatus()==null||sysTenantOneVo.getStatus().equals("")||sysTenantOneVo.getStatus().equals("null")){
            sysTenantOneVo.setStatus("0");
        }
        List<SysTenantTwoVo> list1 = baseMapper.getTenantData(sysTenantOneVo.getTenantName(),
                sysTenantOneVo.getTenantManager(),
                sysTenantOneVo.getTenantType(),
                sysTenantOneVo.getStatus(),
                sysTenantOneVo.getStartTime(),
                sysTenantOneVo.getEndTime(), null, null,sysTenantOneVo.getId());
        int total = 0;
        if (list1.size() > 0) {
            total = list1.size();
        }
        Integer current = null;
        if (sysTenantOneVo.getPage() != null && sysTenantOneVo.getSize() > 0) {
            current = (sysTenantOneVo.getPage() - 1) * sysTenantOneVo.getSize();
        }
        List<SysTenantTwoVo> list = baseMapper.getTenantData(sysTenantOneVo.getTenantName(),
                sysTenantOneVo.getTenantManager(),
                sysTenantOneVo.getTenantType(),
                sysTenantOneVo.getStatus(),
                sysTenantOneVo.getStartTime(),
                sysTenantOneVo.getEndTime(),
                current,
                sysTenantOneVo.getSize(),
                sysTenantOneVo.getId());
        return new CommonPage<>(list, total, sysTenantOneVo.getSize(), sysTenantOneVo.getPage());
    }

    @Override
    public void addTenantData(SysTenant sysTenant) {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        String uuid = UUID.randomUUID().toString();
        uuid = uuid.replace("-", "");
        sysTenant.setTenantCode(uuid);
//        sysTenant.setStatus("0");
        sysTenant.setCreateBy(user.getUserName());
        sysTenant.setCreateTime(LocalDateTime.now());
        this.save(sysTenant);
    }


    @Override
    public void updateTenantData(SysTenant sysTenant) {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        sysTenant.setUpdateBy(user.getUserName());
        sysTenant.setUpdateTime(LocalDateTime.now());
        this.updateById(sysTenant);
    }


    @Override
    public CommonPage<SysTenantTwoVo> getTenantDataOne(String tenantName, String tenantManager, Integer page, Integer size) {
        List<SysTenantTwoVo> list1 = baseMapper.getTenantDataOne(tenantName, tenantManager, null, null);
        int total = 0;
        if (list1.size() > 0) {
            total = list1.size();
        }
        Integer current = null;
        if (page != null && size > 0) {
            current = (page - 1) * size;
        }
        List<SysTenantTwoVo> list = baseMapper.getTenantDataOne(tenantName, tenantManager, current, size);
        return new CommonPage<>(list, total, size, page);
    }
}
