package com.bizmatics.service.system.impl;

import com.bizmatics.common.core.bean.CommonPage;
import com.bizmatics.model.system.SysOperLog;
import com.bizmatics.model.vo.SysOperLogVo;
import com.bizmatics.persistence.mapper.system.SysOperLogMapper;
import com.bizmatics.service.system.ISysOperLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 操作日志 服务层处理
 * 
 * @author yq
 */
@Service
public class SysOperLogServiceImpl implements ISysOperLogService
{
    @Autowired
    private SysOperLogMapper operLogMapper;

    /**
     * 新增操作日志
     * 
     * @param operLog 操作日志对象
     */
    @Override
    public void insertOperlog(SysOperLog operLog)
    {
        operLogMapper.insertOperlog(operLog);
    }

    /**
     * 查询系统操作日志集合
     * 
     * @param operLog 操作日志对象
     * @return 操作日志集合
     */
    @Override
    public List<SysOperLog> selectOperLogList(SysOperLog operLog)
    {
        return operLogMapper.selectOperLogList(operLog);
    }

    /**
     * 批量删除系统操作日志
     * 
     * @param operIds 需要删除的操作日志ID
     * @return 结果
     */
    @Override
    public int deleteOperLogByIds(Long[] operIds)
    {
        return operLogMapper.deleteOperLogByIds(operIds);
    }

    /**
     * 查询操作日志详细
     * 
     * @param operId 操作ID
     * @return 操作日志对象
     */
    @Override
    public SysOperLog selectOperLogById(Long operId)
    {
        return operLogMapper.selectOperLogById(operId);
    }

    /**
     * 清空操作日志
     */
    @Override
    public void cleanOperLog()
    {
        operLogMapper.cleanOperLog();
    }


    @Override
    public CommonPage<SysOperLogVo> getOperLogList(String title, String operName, Integer status, Integer businessType, Integer operId, String startTime, String endTime, Integer page, Integer size) {
        Integer total = operLogMapper.getOperLogCount(title, operName, status, businessType, operId, startTime, endTime);
        Integer current = null;
        if (page != null && size > 0) {
            current = (page - 1) * size;
        }
        List<SysOperLogVo> list = operLogMapper.getOperLogList(title, operName, status, businessType, operId, startTime, endTime, current, size);
        return new CommonPage<>(list, total, size, page);
    }
}
